% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/declare_rs.R
\name{draw_rs}
\alias{draw_rs}
\title{Draw a random sample}
\usage{
draw_rs(declaration = NULL, N = NULL, strata = NULL,
  clusters = NULL, n = NULL, n_unit = NULL, prob = NULL,
  prob_unit = NULL, strata_n = NULL, strata_prob = NULL,
  simple = FALSE, check_inputs = TRUE)
}
\arguments{
\item{declaration}{A random sampling declaration, created by \code{\link{declare_rs}}.}

\item{N}{The number of units. N must be a positive integer. (required)}

\item{strata}{A vector of length N that indicates which stratum each unit belongs to.}

\item{clusters}{A vector of length N that indicates which cluster each unit belongs to.}

\item{n}{Use for a design in which n units (or clusters) are sampled. In a stratified design, exactly n units in each stratum will be sampled. (optional)}

\item{n_unit}{Under complete random sampling, must be constant across units. Under stratified random sampling, must be constant within strata.}

\item{prob}{Use for a design in which either floor(N*prob) or ceiling(N*prob) units (or clusters) are sampled. The probability of being sampled is exactly prob because with probability 1-prob, floor(N*prob) units (or clusters) will be sampled and with probability prob, ceiling(N*prob) units (or clusters) will be sampled. prob must be a real number between 0 and 1 inclusive. (optional)}

\item{prob_unit}{Must of be of length N. Under simple random sampling, can be different for each unit or cluster.  Under complete random sampling, must be constant across units. Under stratified random sampling, must be constant within strata.}

\item{strata_n}{Use for a design in which strata_n describes the number of units to sample within each stratum.}

\item{strata_prob}{Use for a design in which strata_prob describes the probability of being sampled within each stratum. Differs from prob in that the probability of being sampled can vary across strata.}

\item{simple}{logical, defaults to FALSE. If TRUE, simple random sampling is used. When \code{simple = TRUE}, please do not specify n or strata_n. When \code{simple = TRUE}, \code{prob} may vary by unit.}

\item{check_inputs}{logical. Defaults to TRUE.}
}
\description{
You can either give draw_rs() an declaration, as created by \code{\link{declare_rs}} or you can specify the other arguments to describe a random sampling procedure.
}
\examples{
declaration <- declare_rs(N = 100, n = 30)
S <- draw_rs(declaration = declaration)
table(S)

# equivalent to
S <- draw_rs(N = 100, n = 30)
table(S)

}
